#!/bin/bash

#=======================================================
#  日医特定健診ソフトウェア 健診機関番号変更プログラム  
#                   Linux版    最終更新日  2014/12/16   
#
#                   動作環境:      Ubuntu 12.04/10.04
#=======================================================

firebird_path=""
nittoku_path="$HOME/NITTOKU"

fb_username="sysdba"
fb_password="masterkey"


convecho(){
  case $LANG in
  *[uU][tT][fF]*)
    echo -e "$*"
    ;;
  *[eE][uU][cC]*)
    echo -e "$*" | iconv -f utf-8 -t euc-jp
    ;;
  *)
    echo -e "$*"
    ;;
  esac
}


convecho_n(){
  case $LANG in
  *[uU][tT][fF]*)
    echo -e -n "$*"
    ;;
  *[eE][uU][cC]*)
    echo -e -n "$*" | iconv -f utf-8 -t euc-jp
    ;;
  *)
    echo -e -n "$*"
    ;;
  esac
}


welcome_message(){
  convecho "======================================================="
  convecho "  日医特定健診ソフトウェア 健診機関番号変更プログラム  "
  convecho "                   Linux版    最終更新日  2014/12/16   "
  convecho "======================================================="
  convecho "[注意] プログラムの実行前に'必ず'NITTOKUフォルダの     "
  convecho "バックアップを行ってください。                         "
}


check_rootexec(){
  if [ `id -u` -eq 0 ]; then
    convecho "\n[WARNING] このプログラムは通常root権限で動かす必要がありません。"
    convecho "一般ユーザ権限で実行してください。"
    convecho "現在のパス :$(pwd)"
    convecho "実行ユーザ :$(id -un)"
    exit 1
  fi
}

# Tarでのインストール、パッケージFirebird 2.0以上
get_firebird_path(){
  if [ -x "/opt/firebird/bin/isql" ]; then
    firebird_path="/opt/firebird/bin/isql"
  elif [ -x "/usr/bin/isql-fb" ]; then
    firebird_path="/usr/bin/isql-fb"
  fi
}


check_firebird_install(){
  if ! [ -x "$firebird_path" ]; then
    convecho "\n[ERROR] データベースソフトFirebirdがインストールされていないようです。"
    convecho "終了します。"
    exit 2
  fi
}


check_nittoku_install(){
  if ! [ -e "$nittoku_path/JKenshinSoftware.jar" ]; then
    convecho "\n[ERROR] 特定健康診査ソフトウェアが標準の場所 $nittoku_path に"
    convecho "インストールされていないようです。"
    convecho "終了します。"
    exit 3
  fi
}


confirm_start(){
  convecho "\n1. 健診機関番号の変更を開始する"
  convecho "2. 終了する"
  convecho_n "1 または 2 を選択してください :"
  read answer

  case $answer in
  '1')
    ;;
  '2')
    convecho "\nスクリプトの実行を中止しました。終了します。"
    exit 4
    ;;
  *)
    confirm_start
    ;;
  esac
}


get_old_hospnum(){
  convecho_n "\n元の健診機関番号を入力してください (10桁の数字) :"
  read old_hospnum

  if ! [ -e $nittoku_path/DB/$old_hospnum.fdb ]; then
    convecho "\n[ERROR] :該当するデータベースファイルが見つかりません。"
    convecho "健診機関番号を確認した上で再度プログラムを実行してください。終了します。"
    exit 5
  fi
}


get_new_hospnum(){
  convecho_n "\n新しい健診機関番号を入力してください (10桁の数字) :"
  read new_hospnum

  if [ $(expr length $new_hospnum) -ne 10 ]; then
    convecho "\n[ERROR] 健診機関番号が10桁ではありません。"
    convecho "下記で2を選んで修正してください。"
  fi

  if [ -e $nittoku_path/DB/$new_hospnum.fdb ]; then
    convecho "\n[ERROR] :その健診機関番号はすでに利用されています。"
    convecho "データベースファイル名を確認してください。終了します。"
    exit 6
  fi
  
  convecho "\n新しい健診機関番号は「$new_hospnum」で合っていますか?"
  convecho "1. はい、合っています"
  convecho "2. いいえ、修正します"
  convecho_n "1 または 2 を選択してください :"
  read answer

  case $answer in
  '1')
    ;;
  *)
    get_new_hospnum
    ;;
  esac
}


create_sql_files(){
  tempdir=$(mktemp -d)
  
  echo "UPDATE T_F_KIKAN SET TKIKAN_NO = '$new_hospnum' WHERE TKIKAN_NO = '$old_hospnum';" > $tempdir/system.sql
  echo "UPDATE T_KIKAN SET TKIKAN_NO = '$new_hospnum',SMOTO_KIKAN = '$new_hospnum' WHERE TKIKAN_NO = '$old_hospnum';" > $tempdir/kikan.sql
}


exec_sql_files(){
  convecho "----------------------------------------------------"

  $firebird_path -user $fb_username -password $fb_password -input $tempdir/system.sql $nittoku_path/DB/System.fdb
  ret_system=$?

  if [ $ret_system -eq 0 ]; then
    convecho "system.fdb :○"
  else
    convecho "system.fdb :×   エラーコード:$ret_system"
  fi
  

  $firebird_path -user $fb_username -password $fb_password -input $tempdir/kikan.sql $nittoku_path/DB/$old_hospnum.fdb
  ret_kikan=$?

  if [ $ret_kikan -eq 0 ]; then
    convecho "kikan.fdb :○"
  else
    convecho "kikan.fdb :×   エラーコード:$ret_kikan"
  fi
}


move_fdb_file(){
  mv $nittoku_path/DB/$old_hospnum.fdb $nittoku_path/DB/$new_hospnum.fdb
  ret_fdb=$?

  if [ $ret_fdb -eq 0 ]; then
    convecho "DBファイル名の変更 :○"
  else
    convecho "DBファイル名の変更 :×   エラーコード:$ret_fdb"
  fi
}


move_property_file(){
  if [ -e "$nittoku_path/orca_$old_hospnum.properties" ]; then
    mv $nittoku_path/orca_$old_hospnum.properties $nittoku_path/orca_$new_hospnum.properties
    ret_property=$?

    if [ $ret_property -eq 0 ]; then
      convecho "DBファイル名の変更 :○"
    else
      convecho "DBファイル名の変更 :×   エラーコード:$ret_property"
    fi
  fi
}


end_message(){
  rm -rf $tempdir

  convecho "----------------------------------------------------"

  if [ $(expr $ret_system + $ret_kikan + $ret_fdb + $ret_property) -eq 0 ]; then
    convecho "健診機関番号の変更が完了しました。"
  else
    convecho "健診機関番号の変更でエラーが発生しました。"
    convecho "バックアップを元に戻してください。"
  fi
}


# main
welcome_message
check_rootexec
get_firebird_path
check_firebird_install
check_nittoku_install
confirm_start
get_old_hospnum
get_new_hospnum
create_sql_files
exec_sql_files
move_fdb_file
move_property_file
end_message

